import argparse

from model.logger.spotify_logger import Logger
from model.runners.playlist_creator_runner import PlaylistCreatorRunner

if __name__ == '__main__':
    runner = PlaylistCreatorRunner()
    argparser = argparse.ArgumentParser()
    argparser.add_argument("-s", "--songs_path",
                           default=None,
                           type=str,
                           help="Path to songs in a directory")
    argparser.add_argument("-r", "--artists_list",
                           type=str,
                           default=None,
                           help="Path to artists file list")
    argparser.add_argument("-l", "--albums_list",
                           type=str,
                           help="Path to albums file list")
    argparser.add_argument("-g", "--genres_list",
                           type=str,
                           help="Path to genres file list")
    argparser.add_argument("-d", "--down",
                           type=int,
                           default=0,
                           help="Minimum time for playlist")
    argparser.add_argument("-u", "--up",
                           type=int,
                           default=180,
                           help="Maximum time for playlist")
    argparser.add_argument("-m", "--minimum_songs",
                           type=int,
                           default=0,
                           help="Minimum songs required from input")
    argparser.add_argument("-c", "--country",
                           type=str,
                           default="United States",
                           help="Country name to base playlist on")
    runner.run(argparser.parse_args())
    Logger().done()



from abc import ABC, abstractmethod


class ISaver(ABC):
    """
    Interface for saving data
    """

    @abstractmethod
    def save(self, data):
        """
        Save data in a specific way.
        :param data: data to save
        """
        pass



from gekko import GEKKO


class AvgRatingOptimizer(object):
    """
    A class that wraps GEKKO optimizer.
    """
    def __init__(self, songs):
        """
        :param songs: songs to optimize
        """
        self.songs = songs
        self.m = GEKKO()
        self.songs_vars = [self.m.Var(lb=0, ub=1, integer=True) for i in range(len(songs))]
        self._sum_songs_var = self.m.sum(self.songs_vars)
        self.popularity_params = [self.m.Param(song.Popularity) for song in songs]
        exist_param = [self.m.Param(1) for _ in songs]
        self.avg_popularity = self.m.sum([(song_para * s_var) / self.m.max2(1, self._sum_songs_var)
                                          for s_var, song_para in zip(self.songs_vars, self.popularity_params)])
        duration_params = [self.m.Param(song.Duration) for song in self.songs]
        self.sum_time = self.m.sum([song_para * s_var for s_var, song_para in zip(self.songs_vars, duration_params)])
        self.sum_genre = None
        self.sum_atleast = None
        self._equatations = [self.m.sum([song * exist for song, exist in zip(self.songs_vars, exist_param)]) == self._sum_songs_var]

    def add_min_time_constraint(self, min_time):
        """
        Add minimum time constraint for the playlist
        :param min_time: minimum time
        """
        self._equatations += [self.sum_time >= min_time]

    def add_max_time_constraint(self, max_time):
        """
        Add maximum time constraint for the playlist
        :param max_time: maximum time
        """
        self._equatations += [self.sum_time <= max_time]

    def add_genres_constraint(self, genre, number):
        """
        Add constraint for a genre that should be part of the song
        :param genre: the genre's name
        :param number: number of songs that should be from the genre
        """
        genre_params = [self.m.Param(1) if genre in song.Genres else self.m.Param(0) for song in self.songs]
        self.sum_genre = self.m.sum([s_var * gen for s_var, gen in zip(self.songs_vars, genre_params)])
        self._equatations += [self.sum_genre >= number]

    def add_artists_constraint(self, artists):
        """
        Add a constraint for specific artists that should be in the playlist
        :param artists: artists names
        """
        for artist in artists:
            artist_params = [self.m.Param(1) if artist in song.Artists else self.m.Param(0) for song in
                             self.songs]
            sum_artist = self.m.sum(
                [s_var * art for s_var, art in zip(self.songs_vars, artist_params)])
            self._equatations += [sum_artist >= 1]

    def add_atleast_given_songs(self, songs, number):
        """
        Add a constraint for a specific number of songs that should appear in the playlist
        :param songs: songs that must appear from self.songs
        :param number: number of songs that must appear
        """
        songs_params = [self.m.Param(1) if s in songs else self.m.Param(0) for s in self.songs]
        self.sum_atleast = self.m.sum([s_var * song for s_var, song in zip(self.songs_vars, songs_params)])
        self._equatations += [self.sum_atleast >= number]

    def solve(self):
        """
        Solve the optimization problem
        :return: songs for playlist
        """
        self.m.Equation(self._equatations)
        self.m.options.SOLVER = 1
        self.m.Maximize(self.avg_popularity)
        self.m.solve()
        songs_after_opt = []
        for song, song_var in zip(self.songs, self.songs_vars):
            if song_var.VALUE.value[0] == 1.0:
                songs_after_opt += [song]
        return songs_after_opt



import os
from datetime import datetime
from pathlib import Path
from threading import Lock
from time import time

from model.logger import singleton


@singleton
class Logger(object):
    """
    A logger object to save logs into a file
    """

    def __init__(self):
        logs_path = Path(__file__).parent.parent.parent / "logs"
        if not logs_path.is_dir():
            logs_path.mkdir()
        timed_logs_path = logs_path / datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
        if not logs_path.is_dir():
            logs_path.mkdir()
        timed_logs_path.mkdir()
        self._save_to = timed_logs_path / "log.txt"
        self._done_file = timed_logs_path / "done.txt"
        self._valid_chars = [" ", "-", ":", "\n", "\r", "\\", "."]
        self._save_to.touch()
        self._lock = Lock()
        self._b = "-" * 100
        self._start = time()

    def _write_to_log(self, msg):
        """
        Message to write to log file.

        :param msg: message
        """
        self._lock.acquire()

        self._write_to_file(msg)

        self._lock.release()

    def info(self, msg, add_start=True):
        """
        Write an info message to screen and log file.

        :param msg: message
        """
        msg = "".join(e for e in msg if e.isalnum() or e in self._valid_chars)
        if add_start:
            msg = f"Playlist Creator - {datetime.now().strftime('%Y-%m-%d_%H-%M-%S')}: {msg}"
        self._write_to_log(msg)
        print(msg)

    def error(self, msg):
        """
        Write an error message to screen and log file.

        :param msg: message
        """
        msg = "".join(e for e in msg if e.isalnum() or e in self._valid_chars)
        msg = f"Playlist Creator - {datetime.now().strftime('%Y-%m-%d_%H-%M-%S')}: -------- Exception -------- {msg}"
        self._write_to_log(msg)
        print(msg)

    def _write_to_file(self, msg):
        """
        Write a message to log file char by char to get all chars that can be written to the file.

        :param msg: message
        """
        log_file = self._save_to.open("a")
        for c in msg:
            try:
                log_file.write(c)
            except Exception:
                continue
        log_file.write("\n")
        log_file.close()

    def done(self):
        """
        Creating a file to mark the playlist creation as done
        """
        self.beautiful_info(f"Finished in {(time() - self._start) / 60} minutes ({datetime.now().strftime('%Y-%m-%d_%H-%M-%S')})")
        self._done_file.touch()

    def beautiful_info(self, msg):
        """
        Beautiful print for log in format:
        -----------------------
        -----------------------
                  msg
        -----------------------
        -----------------------
        :param msg: message to print
        """
        self.info(self._b, add_start=False)
        self.info(self._b, add_start=False)
        self.info(msg, add_start=False)
        self.info(self._b, add_start=False)
        self.info(self._b, add_start=False)



from model.optimizer.avg_rating_optimizer import AvgRatingOptimizer
from model.music_objs.song import Song
from model.music_objs.artist import Artist
def test_optimizer():
    artist1 = {
        "name": "Metalica"
    }
    #artist1 = Artist(artist1)
    artist2 = {
        "name": "Imagine dragons"
    }
    #artist2 = Artist(artist2)
    artist3 = {
        "name": "Sia"
    }
    #artist3 = Artist(artist3)
    artist4 = {
        "name": "David Guetta"
    }
    #artist4 = Artist(artist4)
    artist5 = {
        "name": "Metalica"
    }
    #artist5 = Artist(artist5)
    song1 = {
        "name": "nothing else matter",
        "artists": [artist1],
        "duration_ms": 50000,
        "genres": "metal",
        "popularity": 10
    }

    song2 = {
        "name": "Thunder",
        "artists": [artist2],
        "duration_ms": 50000,
        "genres": "pop",
        "popularity": 10
    }
    song3 = {
        "name": "Cheap Thrills",
        "artists": [artist3],
        "duration_ms": 50000,
        "genres": "pop",
        "popularity": 10
    }
    song4 = {
        "name": "Titanium",
        "artists": [artist4],
        "duration_ms": 50000,
        "genres": "sol",
        "popularity": 10
    }
    song5 = {
        "name": "best",
        "artists": [artist5],
        "duration_ms": 150000,
        "genres": "metal",
        "popularity": 30
    }
    song1 = Song(song1)
    song2 = Song(song2)
    song3 = Song(song3)
    song4 = Song(song4)
    song5 = Song(song5)
    songs = [song1,song2,song3,song4,song5]
    opt = AvgRatingOptimizer(songs)
    opt.add_max_time_constraint(200)
    opt.add_min_time_constraint(100)
    # opt.add_genres_constraint("pop",1)
    # opt.add_genres_constraint("metal", 1)
    # opt.add_artists_constraint([artist1,artist2])
    opt.add_atleast_given_songs([song1,song2,song3,song4],3)
    sulotion = opt.solve()
    print('\n'.join([song.Name for song in sulotion]))
    assert True

test_optimizer()




import time
from collections import defaultdict
from threading import Thread, Lock

import numpy as np
from dateutil.parser import parse

from model.logger.spotify_logger import Logger
from model.optimizer.avg_rating_optimizer import AvgRatingOptimizer
from model.playlist_creator.playlist_creator_interface import IPlaylistCreator
from model.playlist_creator.playlist_modes import PlaylistModes


class PlaylistCreatorBase(IPlaylistCreator):
    """
    A class that gets a music song and creates a playlist from it in various ways.
    """

    def __init__(self, music_searcher, mode=PlaylistModes.SONGS, num_of_can=50, country="US"):
        """
        :param music_searcher: A IMusicSearcher object
        :param mode: mode to create a playlist by
        """
        self._mode = mode
        self._music_searcher = music_searcher
        self.logger = Logger()
        self._num_of_can = num_of_can
        self._country = country

    def create_playlist(self, music_source, **kwargs):
        """
        Create a playlist from a given music source.

        :param music_source: music source
        :param kwargs: any argument needed for a specific mode
        :return: playlist
        """
        if self._mode == PlaylistModes.SONGS:
            return self._songs_creator(music_source, **kwargs)
        elif self._mode == PlaylistModes.ARTISTS:
            return self._artists_creator(music_source, **kwargs)
        elif self._mode == PlaylistModes.GENRES:
            return self._genres_creator(music_source, **kwargs)
        elif self._mode == PlaylistModes.ALBUMS:
            return self._albums_creator(music_source, **kwargs)

    def _songs_creator(self, songs, genres=None, min_time=0, max_time=2400,
                       append=True, artists=None, num_of_songs=0):
        """
        Create a playlist from songs

        :param songs: songs to create a playlist from
        :param genres: genres wanted for the songs
        :param min_time: minimum time required for a playlist
        :param max_time: maximum time required for the playlist
        :param append: append similar songs
        :return:
        """
        self.logger.info("Creating playlist by songs...")
        prev_song_len = len(songs)
        if append:
            self.logger.info("Appending all similar songs")
            self._append_all_similar_songs(songs)
            self.logger.info(f"Finished appending all similar songs, got {len(songs)} songs")
            songs = self._get_similar(songs)

        PlaylistCreatorBase._add_weight_to_songs(songs)

        songs = PlaylistCreatorBase._optimize_weight(songs)

        return PlaylistCreatorBase._optimize_popularity(songs, min_time, max_time,
                                                        genres, artists, songs[:prev_song_len], num_of_songs)

    def _append_all_similar_songs(self, songs):
        """
        Get more similar songs from given songs.

        :param songs: songs
        :return: appended list of songs
        """
        lock = Lock()

        def set_simillar_songs(name, artist, songs_list):
            res = self._music_searcher.get_similar_tracks(name, artist, num_of_similar=50)
            self.logger.info(f"Adding similar for: {artist} - {name}")
            lock.acquire()
            songs_list += res
            lock.release()

        PlaylistCreatorBase._run_in_thread_loop(songs, target=set_simillar_songs,
                                                args_method=lambda song, songs_list: (song.Name, song.Artists[0],
                                                                                      songs_list))

    @staticmethod
    def _add_weight_to_songs(songs):
        """
        Add weight to songs by their popularity and by the artists that are in the song

        :param songs: Songs list
        """
        Logger().info("Calculating weights for songs...")
        same_artists = {}
        for song in songs:
            for artist in song.Artists:
                if artist not in same_artists:
                    same_artists[artist] = []
                same_artists[artist] += [song]

        Logger().info("Setting weights for songs...")
        PlaylistCreatorBase._set_weights(same_artists)
        Logger().info("Setting weights for songs is done.")

    @staticmethod
    def _set_weights(same_artists):
        """
        Set the weight for the songs by a given dict of artist -> Songs list.

        :param same_artists: dict
        """
        for val in same_artists.values():
            val.sort(key=lambda x: x.Popularity, reverse=True)
            val_len = len(val)
            for i in range(val_len):
                val[i].set_weight(val[i].Weight + ((val_len - i) / val_len) * val[i].Popularity)

    @staticmethod
    def _optimize_weight(songs):
        """
        Get optimal weighted songs.

        :param songs: list of Songs
        :return: optimal weighted songs
        """
        if len(songs) < 30:
            return songs
        songs.sort(key=lambda x: x.Weight, reverse=True)
        third = int(len(songs) * (1 / 3))
        return songs[:third]

    @staticmethod
    def _optimize_popularity(songs, min_time, max_time, genres, artists, initial_songs, num_of_songs):
        """
        Optimize songs by their popularity.

        :param songs: songs list
        :param min_time: min time for playlist
        :param max_time: max time for playlist
        :param genres: required genres
        :param initial_songs: songs before append
        :param num_of_songs: at number of songs to take from initial_songs
        :return: indexes of songs to take to playlist
        """
        optimizer = AvgRatingOptimizer(songs)
        optimizer.add_min_time_constraint(min_time)
        optimizer.add_max_time_constraint(max_time)
        if genres:
            for genre, num in genres.items():
                optimizer.add_genres_constraint(genre, num)
        if artists:
            optimizer.add_artists_constraint(artists)
        if num_of_songs:
            optimizer.add_atleast_given_songs(initial_songs, num_of_songs)
        return optimizer.solve()

    @classmethod
    def _get_songs_after_optimization(cls, songs_vars, songs):
        """
        Get songs chosen by optimizer.

        :param songs_vars: list of 1s and 0s (1 if to take the song and 0 O.W.)
        :param songs: Songs list
        :return: songs after filtering
        """
        songs_after_opt = []
        for song, song_var in zip(songs, songs_vars):
            if song_var.VALUE.value[0] == 1.0:
                songs_after_opt += [song]
        return songs_after_opt

    def _artists_creator(self, artists, **kwargs):
        """
        Create a playlist by artists.

        :param artists: list of Artists
        :param kwargs: any argument required for creating playlist by songs
        :return: playlist
        """
        top_tracks = self._get_top_tracks(artists)
        artists_names = [ar.Name for ar in artists]

        more_artists = self._get_more_artists(top_tracks, artists_names)

        actual_artists = self._get_similar(more_artists)

        top_tracks += self._get_top_tracks(actual_artists)
        self.logger.info(f"Got songs top tracks for artists, got {len(top_tracks)} songs")

        PlaylistCreatorBase._set_weight_by_artist(top_tracks, artists)

        return self._songs_creator(top_tracks, append=False, **kwargs)

    def _get_top_tracks(self, artists):
        """
        Get top tracks by given list of artists.

        :param artists: Artists list
        :return: top tracks for all artists
        """
        lock = Lock()

        def set_top_tracks(name):
            res = self._music_searcher.get_artists_top_tracks(name, country=self._country)
            Logger().info(f"Adding similar for: {name}")
            lock.acquire()
            tracks = set_top_tracks.__getattribute__("tracks")
            tracks += res
            lock.release()

        set_top_tracks.__setattr__("tracks", [])
        PlaylistCreatorBase._run_in_thread_loop(artists, target=set_top_tracks,
                                                args_method=lambda artist, _: (artist.Name,))

        return set_top_tracks.__getattribute__("tracks")

    def _get_more_artists(self, top_tracks, artists_names):
        """
        Get more artists info from top tracks.

        :param top_tracks: top tracks list
        :param artists_names: artists list
        :return: list of Artists
        """
        more_artists = []
        for track in top_tracks:
            for artist in track.Artists:
                if artist not in artists_names:
                    artists_names += [artist]
                    more_artists += [self._music_searcher.get_artist_info(artist)]
        return more_artists

    @staticmethod
    def _set_weight_by_artist(top_tracks, artists):
        """
        Set weight for tracks by number of follower of the artists that sing in the track.

        :param top_tracks: top tracks list
        :param artists: Artists list 
        """
        artists_followers_dict = {artist.Name: artist.NumOfFollowers for artist in artists}
        total_sum_of_followers = sum([artist.NumOfFollowers for artist in artists])

        for track in top_tracks:
            for artist in track.Artists:
                if artist in artists_followers_dict:
                    track.set_weight(track.Weight + (artists_followers_dict[artist] / total_sum_of_followers))

    def _genres_creator(self, genres, **kwargs):
        """
        Create a playlist by genres.

        :param genres: genres to get songs from
        :param kwargs: any argument needed to optimize by songs
        :return: playlist
        """
        i, n = 0, len(genres)
        genres_keys = [gen for gen in genres.keys()]
        songs = []
        while i < n:
            three = []
            for j in range(i, min(i + 3, n)):
                three += [genres_keys[j]]
            i += 3
            songs += self._music_searcher.get_songs_by_genres(genres=three)
        self.logger.info(f"Got songs for genres, got {len(songs)} songs")
        return self._songs_creator(songs, append=False, genres=genres, **kwargs)

    def _albums_creator(self, albums, **kwargs):
        """
        Create playlist from albums.

        :param albums: Albums list
        :param kwargs: arguments
        :return: playlist
        """
        albums = self._get_similar(albums)
        songs = []
        for album in albums:
            songs += album.Tracks
        self.logger.info(f"Got songs for albums, got {len(songs)} songs")
        return self._songs_creator(songs, append=False, **kwargs)

    @staticmethod
    def _run_in_thread_loop(list_to_run, target, args_method):
        """
        Run in threads an operation (target) on each item in the list.

        :param list_to_run: list to run operations on each element
        :param target: operation to run
        :param args_method: method to get args for the operation
        """
        threads = []
        for item in list_to_run:
            th = Thread(target=target, args=args_method(item, list_to_run))
            th.start()
            threads += [th]

        for t in threads:
            t.join(timeout=2400)
            Logger().info("Still waiting to finish...")

    def _get_similar(self, music_source):
        """
        Get similar music sources as part of the Candidate Generation step.

        :param music_source: music data
        :return: candidates
        """
        self.logger.info("Doing Candidate Generation step")
        if self._mode == PlaylistModes.GENRES or len(music_source) < self._num_of_can:
            return music_source

        genres_counts = defaultdict(lambda: [0, None])
        i = 0
        for music in music_source:
            for genre in music.Genres:
                genres_counts[genre][0] += 1
                if not genres_counts[genre][1]:
                    genres_counts[genre][1] = i
                    i += 1

        if self._mode == PlaylistModes.SONGS:
            return self._get_candidates_for_songs(music_source, genres_counts)
        if self._mode == PlaylistModes.ARTISTS:
            return self._get_candidates_for_artists(music_source, genres_counts)
        if self._mode == PlaylistModes.ALBUMS:
            return self._get_candidates_for_albums(music_source, genres_counts)

    def _get_candidates_for_songs(self, songs, genres_counts):
        """
        Get candidates by songs.

        :param songs: songs data
        :param genres_counts: count of generes
        :return: candidates
        """
        num_of_cat = len(genres_counts)
        num_of_music = len(songs)
        vectors = np.zeros((num_of_music, num_of_cat + 1))
        for music, vec in zip(songs, vectors):
            vec[0] = music.Popularity
            for genre in music.Genres:
                cost, index = genres_counts[genre]
                if (index + 1) < (num_of_cat + 1):
                    vec[index + 1] = cost

        return self._get_similar_from_vecs(vectors, songs, num_of_music)

    def _get_candidates_for_artists(self, artists, genres_counts):
        """
        Get candidates by artists.

        :param artists: songs data
        :param genres_counts: count of generes
        :return: candidates
        """
        num_of_cat = len(genres_counts)
        num_of_music = len(artists)
        vectors = np.zeros((num_of_music, num_of_cat + 2))
        for music, vec in zip(artists, vectors):
            vec[0] = music.Popularity
            vec[1] = music.NumOfFollowers
            for genre in music.Genres:
                cost, index = genres_counts[genre]
                vec[index + 2] = cost

        return self._get_similar_from_vecs(vectors, artists, num_of_music)

    def _get_candidates_for_albums(self, albums, genres_counts):
        """
        Get candidates by albums.

        :param albums: songs data
        :param genres_counts: count of genres
        :return: candidates
        """
        num_of_cat = len(genres_counts)
        num_of_music = len(albums)
        vectors = np.zeros((num_of_music, num_of_cat + 4))
        times = []
        for music, vec in zip(albums, vectors):
            vec[0] = music.Popularity
            vec[1] = music.NumOfTracks
            vec[2] = music.AvgTracksPopularity
            album_time = parse(music.ReleaseDate)
            actual_time = time.mktime(album_time.timetuple())
            times += [actual_time]
            vec[3] = actual_time
            for genre in music.Genres:
                cost, index = genres_counts[genre]
                vec[index + 4] = cost
        times = np.array(times)
        mean, std = times.mean(), times.std()
        for vec in vectors:
            vec[3] = (vec[3] - mean) / std

        return self._get_similar_from_vecs(vectors, albums, num_of_music)

    def _get_similar_from_vecs(self, vectors, music_source, num_of_music):
        """
        Get candidates by doing evaluating the dot product for all music sources.

        :param vectors: vectors after weighting
        :param music_source: the music source
        :param num_of_music: number of music objects
        :return: candidates
        """
        for i in range(num_of_music):
            mat_without_i = np.array(vectors[:i].tolist() + vectors[i + 1:].tolist())
            music_source[i].set_similarity((mat_without_i @ vectors[i].T).sum())

        music_source.sort(key=lambda m: m.Similarity)
        return music_source[:self._num_of_can]



from pathlib import Path

from mutagen.mp3 import MP3
from mutagen.wave import WAVE

from model.logger.spotify_logger import Logger


class DirectoryReader:
    """
    Class to read the contents of a songs directory
    """
    def __init__(self):
        self._logger = Logger()

    def get_files(self, p):
        """
        Get file from songs folder
        :param p: path to directory
        :return: files in the directory
        """
        files = []
        self._logger.info(f"Getting data from songs files from folder {str(p)}")
        for x in p.iterdir():
            self._logger.info(f"Current file or dir: {x}")
            if x.is_dir():
                files.extend(self.get_files(x))
            elif x.suffix == '.mp3' or x.suffix == '.wav':
                files.append(x)
        self._logger.info(f"Finished getting data from songs folder: {str(p)}")
        return files

    def get_songs(self, path):
        """
        Get songs from path.

        :param path: path for songs directory
        :return: songs in path
        """
        self._logger.info(f"Getting songs from path: {path}")
        p = Path(path)
        files = self.get_files(p)
        songs = []
        for file in files:
            songs.append(self.get_song_attr(file))
        self._logger.info(f"FINISHED GETTING SONGS")
        return songs

    def get_song_attr(self, file):
        """
        Gets the attributes needed from the songs.

        :param file: the path to the file
        :return: attributes of the file
        """
        song = {}
        self._logger.info(f"Getting attributes for the file: {file}")
        if file.suffix == '.mp3':
            audio = MP3(file)
        else:
            audio = WAVE(file)
        desc = file.name.rsplit('.', 1)[0]
        split = desc.rsplit('-', 1)
        artist, name = split[0] if len(split) > 1 else '', split[1] if len(split) > 1 else split[0]
        if artist == '':
            artist = str(audio.tags['TPE1'])
        name = name.split("ft")[0].strip()
        name = name.split("feat")[0].strip()
        song['name'], song['artist'], song['duration'] = name, artist.strip(), audio.info.length
        return song



import datetime
import logging
from pathlib import Path


from model.playlist_creator.playlist_creator_base import PlaylistCreatorBase
from model.playlist_creator.playlist_modes import PlaylistModes
from model.songs_directory.directory_reader import DirectoryReader
from model.songs_searchers.spotify_searcher import SpotifySearcher

from gekko import GEKKO


def test_last_fm_searcher():
    dr = DirectoryReader()
    songs = dr.get_songs("C:\\Users\\Idan Cohen\\Desktop\\songs")
    songs_objs = []
    client_id = '03e69fa5ec5d479a82bb066e019a722b'
    client_secret = '0ae6f5bb823a4ac281dc1b7c07180706'
    searcher = SpotifySearcher(client_id, client_secret)
    res_s, res_a = [], []
    s = searcher.get_song_info("too much to ask", "niall horan")
    res_s += [s]

    p = PlaylistCreatorBase(searcher, mode=PlaylistModes.SONGS)
    res = p.create_playlist(res_s, min_time=1000, max_time=2400)
    assert True

    #
    # song_ids = [s.ID for s in res_s]
    # art_ids = [s.ID for s in res_a]
    #
    # res = searcher.get_similar_tracks(song["name"], song["artist"])
    # r = res[0].Duration
    # songs = ["x1", "x2", "x3", "x4", "x5"]
    # ratings = [40, 50, 60, 70, 80]
    # times = [300, 200, 130, 44, 123]
    #
    # m = GEKKO()  # create GEKKO model
    # songs_vars = [m.Var(lb=0, ub=1, integer=True) for i in range(5)]
    # songs_vars_sum = m.sum(songs_vars)
    # weight_params = [1,2,3,4,5]
    # weight = m.sum([(song_para * s_var) for s_var, song_para in zip(songs_vars, weight_params)])
    # m.Equation([songs_vars_sum <= 30, ])
    # m.Maximize(weight)
    # m.solve()
    # print(1)
    # s = [m.Var(lb=0, ub=1, integer=True) for i in range(5)]  # define new variable, default=0
    # tt = m.sum([s * m.Param(t) for s, t in zip(s, times)])
    # rr = m.sum([p * m.Param(r) / max(1, sum([v.value for v in s])) for p, r in zip(s, ratings)])
    # m.Equations([tt >= 100, tt <= 200])  # equations
    # m.Maximize(rr)
    # m.options.SOLVER = 1  # APOPT solver
    # m.solve()
    # print(1)
    # s = pulp.LpVariable.dicts("songs", songs, 0, 1, LpBinary)
    # r_s = pulp.lpSum((son * r) / max(1, sum([1 for p in s.values() if p.value()])) for son, r in zip(s.values(),
    #                                                                                                  ratings))
    # t_s = pulp.lpSum(t * son for son, t in zip(s.values(), times))
    #
    # prob = LpProblem("MMM", LpMaximize)
    # prob += r_s
    # prob += t_s >= 100
    # prob += t_s <= 200
    # r = prob.solve()


test_last_fm_searcher()



import json
from pathlib import Path

import pylab as plt
import xlwt

from model.savers.saver_interface import ISaver


class PlaylistSaver(ISaver):
    """
    A playlist saver for Song objects.
    Saves the playlist as a json file.
    """

    def __init__(self, name="playlist.json"):
        """
        :param name: name for the file that the playlist will be save to
        """
        if not name.endswith(".json"):
            name += ".json"
        self._name = name
        self._playlist_folder = Path(__file__).parent.parent.parent / "playlists"

    def save(self, songs):
        """
        Get a list of songs and save them to a json file.

        :param songs: list of Songs
        """
        playlist_result_path = self._playlist_folder / self._name
        if not self._playlist_folder.is_dir():
            self._playlist_folder.mkdir()
        result_dict = {
            "songs": {}
        }
        cols = ["ID", "Popularity", "Link"]
        y = []
        x = []
        rows = []
        table = []
        total_time_in_minute = 0
        n = 0
        for song in songs:
            table += [[n, song.Popularity, song.LinkToSong]]
            y += [song.Popularity]
            n += 1
            x += [n]
            ar = song.Artists[0].replace("$", "")
            if f"{ar} - {song.Name}" not in rows:
                rows += [f"{ar} - {song.Name}"]
                result_dict["songs"][song.ID] = {
                    "artists": song.Artists,
                    "name": song.Name,
                    "link": song.LinkToSong,
                    "popularity": song.Popularity
                }
                total_time_in_minute += song.Duration

        result_dict["total_time_in_minute"] = total_time_in_minute / 60
        result_json = json.dumps(result_dict, indent=4, sort_keys=True)
        playlist_result_path.write_text(result_json)

        self._show_table_graph(x, y)
        self._save_to_excel(table, rows, cols)

    def _show_table_graph(self, x, y):
        """
        Show the table and graph of the playlist created.
        The graph is the id of the song to its popularity

        :param x: ids of the songs
        :param y: popularities
        """
        g = plt.figure(2)
        plt.plot(x, y)
        plt.ylabel("Songs Popularity")
        plt.xlabel("Songs IDs")
        g.savefig(str(self._playlist_folder / "playlist_graph.pdf"))

    def _save_to_excel(self, table, rows, cols):
        book = xlwt.Workbook(encoding='utf8')
        sheet = book.add_sheet("playlist")

        style = xlwt.easyxf('pattern: pattern solid, fore_colour light_blue;'
                            'font: colour white, bold True;')

        style_c = xlwt.easyxf('pattern: pattern solid, fore_colour light_yellow;'
                              'font: bold True;')
        max_widths = [0] * (len(cols) + 1)
        for i, col in zip(range(len(cols)), cols):
            max_widths[i + 1] = max(max_widths[i + 1], len(str(col)))
            sheet.col(i + 1).width = max_widths[i + 1] * 500
            sheet.write(0, i + 1, col, style)

        for i, row, t in zip(range(1, len(rows) + 2), rows, table):
            max_widths[0] = max(max_widths[0], len(str(row)))
            sheet.col(0).width = max_widths[0] * 500
            sheet.write(i, 0, row, style)
            for j, t_val in zip(range(1, len(t) + 1), t):
                max_widths[j] = max(max_widths[j], len(str(t_val)))
                sheet.col(j).width = max_widths[j] * 500
                sheet.write(i, j, t_val, style_c)

        book.save(str(self._playlist_folder / "playlist_table.xls"))



class Playlist(object):
    def __init__(self, obj):
        self._obj = obj

    @property
    def Name(self):
        return self._obj["name"]

    @property
    def ID(self):
        return self._obj["id"]

    @property
    def Href(self):
        return self._obj["href"]

    @property
    def Genres(self):
        return self._obj["genres"]

    @property
    def Tracks(self):
        return self._obj["songs"]

    @property
    def NumOfTracks(self):
        return self._obj["tracks"]["total"]

    @property
    def LinkToAlbum(self):
        return self._obj["external_urls"]["spotify"]

    @property
    def NumOfFollowers(self):
        return self._obj["followers"]["total"]

    @property
    def AvgTracksPopularity(self):
        return sum([track.Popularity for track in self.Tracks]) / self.NumOfTracks
